function x = proffsub(R,prof,x)
% ----------------------------------------------------------------------
% In-place forward substitution, solving R'*x = b for an upper
% triangular profile (skyline) Cholesky factor R.  R can be stored as a
% dense nxn matrix, as a band matrix or as a fully packed profile
% matrix.  In each case, prof specifies its profile shape. See
% profchol() for details. x is a dense nxk matrix, k>=1.
% ----------------------------------------------------------------------

if nargin<3, error('proffsub: missing arguments'); end
[b,n] = size(R);
nx = size(x,1);

% Skip any leading zeros in x.
for px=1:nx
   if any(x(px,:)), break; end
end

if b==n		% unpacked nxn matrix

   for i=px:n
      J = max(px,prof(i)):i-1;
      x(i,:) = (x(i,:) - R(J,i)'*x(J,:)) / R(i,i);
   end

elseif b<n		% packed band matrix

   % R(prof(i):b,i) represents column i elements A(i+prof(i)-b:i,i) of
   % unpacked matrix.
   for i=px:n
      J = max(px,prof(i)+i-b):i-1;
      x(i,:) = (x(i,:) - R(J-(i-b),i)'*x(J,:)) / R(b,i);
   end

elseif n==1			% fully packed profile matrix
   n = size(prof,2);

   % R(prof(i-1)+1:prof(i)) represents column i elements
   % A(prof(i-1)+1-prof(i)+i,i) of unpacked matrix. di = location of
   % A(i,i) in R, ci = nominal offset of element 0 of column i in R, pi
   % = index of first nonzero element in column i of A. 

   if px>1, diprev = prof(px-1); else diprev = 0; end
   for i=px:n
      di = prof(i);
      ci = di-i;
      pi = diprev+1-ci;
      J = max(px,pi):i-1;
      x(i,:) = (x(i,:) - R(ci+J,1)'*x(J,:)) / R(di);
      diprev = di;
   end
   
else
   error('proffsub: unkonwn matrix format');
end
